@@ -5,7 +5,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2014	PgPool Global Development Group
+ * Copyright (c) 2003-2015	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -97,7 +97,6 @@ static int process_backend_health_check_failure(int health_check_node_id, int re
 static bool do_health_check(bool use_template_db, volatile int *health_check_node_id);
 
 static void FileUnlink(int code, Datum path);
-static int write_status_file();
 static pid_t pcp_fork_a_child(int unix_fd, int inet_fd, char *pcp_conf_file);
 static pid_t fork_a_child(int *fds, int id);
 static pid_t worker_fork_a_child(void);
@@ -401,6 +400,9 @@ int PgpoolMain(bool discard_status, bool clear_memcache_oidmaps)
 	}
 	/* We can now handle ereport(ERROR) */
 	PG_exception_stack = &local_sigjmp_buf;
+
+	/* Create or write status file */
+	(void)write_status_file();
 	
 	/* This is the main loop */
 	for (;;)
@@ -453,15 +455,13 @@ int PgpoolMain(bool discard_status, bool clear_memcache_oidmaps)
 			all_nodes_healthy = do_health_check(use_template_db,&health_check_node_id);
 			POOL_SETMASK(&BlockSig);
 
+			if (all_nodes_healthy && retrycnt)
+				ereport(LOG,
+					(errmsg("all backends are returned to a healthy state after %d retry(ies)",retrycnt)));
+
 			health_check_node_id = 0;
 			use_template_db = false;
 			retrycnt = 0;
-
-			if (all_nodes_healthy && retrycnt)
-				ereport(LOG,
-					(errmsg("after retry %d, All backends are returned to healthy state",retrycnt)));
-
-
 			processState = SLEEPING;
 
 			if (pool_config->health_check_timeout > 0)
@@ -1472,6 +1472,7 @@ static void failover(void)
 					 BACKEND_INFO(node_id).backend_port)));
 
 			BACKEND_INFO(node_id).backend_status = CON_CONNECT_WAIT;	/* unset down status */
+			(void)write_status_file();
 
 			/* wait for failback command lock or to be lock holder */
 			if (pool_config->use_watchdog && !wd_am_I_lock_holder())
@@ -1526,6 +1527,8 @@ static void failover(void)
 							 BACKEND_INFO(node_id_set[i]).backend_port)));
 
 					BACKEND_INFO(node_id_set[i]).backend_status = CON_DOWN;	/* set down status */
+					(void)write_status_file();
+
 					/* save down node */
 					nodes[node_id_set[i]] = 1;
 					cnt++;
@@ -1724,6 +1727,8 @@ static void failover(void)
 									 bkinfo->backend_hostname,
 									 bkinfo->backend_port)));
 							bkinfo->backend_status = CON_DOWN;	/* set down status */
+							(void)write_status_file();
+
 							follow_cnt++;
 						}
 					}
@@ -2406,7 +2411,7 @@ static int trigger_failover_command(int node, const char *command_line,
 							string_append_char(exec_cmd, newmaster->backend_hostname);
 						else
 							/* no valid new master */
-							string_append_char(exec_cmd, "");
+							string_append_char(exec_cmd, "\"\"");
 						break;
 
 					case 'm': /* new master node id */
@@ -2423,7 +2428,7 @@ static int trigger_failover_command(int node, const char *command_line,
 						}
 						else
 							/* no valid new master */
-							string_append_char(exec_cmd, "");
+							string_append_char(exec_cmd, "\"\"");
 						break;
 
 					case 'R': /* new master database directory */
@@ -2432,7 +2437,7 @@ static int trigger_failover_command(int node, const char *command_line,
 							string_append_char(exec_cmd, newmaster->backend_data_directory);
 						else
 							/* no valid new master */
-							string_append_char(exec_cmd, "");
+							string_append_char(exec_cmd, "\"\"");
 						break;
 
 					case 'M': /* old master node id */
@@ -2690,6 +2695,13 @@ static void initialize_shared_mem_objects(bool clear_memcache_oidmaps)
 	memset(con_info, 0, size);
 
 	size = pool_config->num_init_children * (sizeof(ProcessInfo));
+
+	ereport(DEBUG1,
+			(errmsg("ProcessInfo: num_init_children (%d) * sizeof(ProcessInfo) (%zu) = %d bytes requested for shared memory",
+					pool_config->num_init_children,
+					sizeof(ProcessInfo),
+					size)));
+
 	process_info = pool_shared_memory_create(size);
 	memset(process_info, 0, size);
 
@@ -2701,6 +2713,10 @@ static void initialize_shared_mem_objects(bool clear_memcache_oidmaps)
 	/* create fail over/switch over event area */
 	Req_info = pool_shared_memory_create(sizeof(POOL_REQUEST_INFO));
 
+	ereport(DEBUG1,
+			(errmsg("Request info are: sizeof(POOL_REQUEST_INFO) %zu bytes requested for shared memory",
+					sizeof(POOL_REQUEST_INFO))));
+
 	/*
 	 * Initialize backend status area.
 	 * From now on, VALID_BACKEND macro can be used.
@@ -2720,6 +2736,10 @@ static void initialize_shared_mem_objects(bool clear_memcache_oidmaps)
 	InRecovery = pool_shared_memory_create(sizeof(int));
 	*InRecovery = RECOVERY_INIT;
 
+	ereport(DEBUG1,
+			(errmsg("Recovery management area: sizeof(int) %zu bytes requested for shared memory",
+					sizeof(int))));
+
 	/*
 	 * Initialize shared memory cache
 	 */
@@ -2820,6 +2840,7 @@ static int read_status_file(bool discard_status)
 			if (backend_rec.status[i] == CON_DOWN)
 			{
 				BACKEND_INFO(i).backend_status = CON_DOWN;
+				(void)write_status_file();
 				ereport(LOG,
 						(errmsg("read_status_file: %d th backend is set to down status", i)));
 			}
@@ -2827,6 +2848,7 @@ static int read_status_file(bool discard_status)
 					 BACKEND_INFO(i).backend_status == CON_UP)
 			{
 				BACKEND_INFO(i).backend_status = CON_CONNECT_WAIT;
+				(void)write_status_file();
 				someone_wakeup = true;
 			}
 			else
@@ -2909,15 +2931,16 @@ static int read_status_file(bool discard_status)
 		{
 			BACKEND_INFO(i).backend_status = CON_CONNECT_WAIT;
 		}
+		(void)write_status_file();
 	}
 
 	return 0;
 }
 
 /*
-* Write the pid file
+* Write the status file
 */
-static int write_status_file()
+int write_status_file()
 {
 	FILE *fd;
 	char fnamebuf[POOLMAXPATHLEN];
@@ -2980,6 +3003,10 @@ static void reload_config(void)
 		(errmsg("reload config files.")));
     MemoryContext oldContext = MemoryContextSwitchTo(TopMemoryContext);
 	pool_get_config(conf_file, RELOAD_CONFIG);
+
+	/* Realoading config file could change backend status */
+	(void)write_status_file();
+
     MemoryContextSwitchTo(oldContext);
 	if (pool_config->enable_pool_hba)
 		load_hba(hba_file);
@@ -3014,7 +3041,7 @@ static void system_will_go_down(int code, Datum arg)
     }
     POOL_SETMASK(&AuthBlockSig);
     /* Write status file */
-    write_status_file();
+    (void)write_status_file();
     /* 
      * Terminate all childrens. But we may already have killed
      * all the childrens if we come to this function because of shutdown
